/*
 * $Id: gmx_puckering.c 22 2009-06-08 15:21:24Z revilo $
 *
 *                This source code is part of
 *
 *                        g_puckering
 *
 * g_puckering is an analysis-tool for calculating the Cremer-Pople
 * ring puckering parameters within a given GROMACS trajectory file.
 *
 * written 2007 by Oliver Stueker
 *
 * Acknowledgments:
 *  The Analysis tool bases on a template and several functions from:
 *  GROMACS - GROningen MAchine for Chemical Simulations
 *  an open source program to perform molecular dynamics simulations
 *  under GNU General Public License.
 *  Written by David van der Spoel, Erik Lindahl, Berk Hess, and others.
 *  check out http://www.gromacs.org for more information
 *
 *  The implementation of Cremer-Pople-Puckering Algorithm
 *  was done by Mark J. Forster for his GPL-program mdxvu
 *  http://sourceforge.net/projects/mdxvu/
 *
 * License:
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */
#include <string.h>
#include <stdio.h>
#include <math.h>
#include "macros.h"
#include "statutil.h"
#include "xvgr.h"
#include "gmx_puckering.h"

void writeTimePlot(char *fn, int type, char *grpnm, struct SFrame *firstFrame) {
  /**
   * fn       filename
   * type     plotType
   * grpnm    Name of Indexgroup
   * firstFrame pointer to Linked List
   */

  /* TODO: Introduce option to plot Q in Angstroem */

  char  sTitle[STRLEN];
  char  sXlabel[STRLEN];
  char  sYlabel[STRLEN];
  int   nFrames;
  float endTime;
  float fXmax;
  float fYmax=360.0;
  FILE  *out;

  struct SFrame *currentFrame;
  struct SRing  *currentRing;

  currentFrame=firstFrame;
  while (1) {
    if (currentFrame->next)
      currentFrame=currentFrame->next;
    else {
      endTime=currentFrame->fTime;
      nFrames=currentFrame->nNumber+1;
      break;
    }
  }
  fXmax=endTime*1.05;

  switch (type) {
  case ePlotTHETA:
    sprintf(sTitle, "CP Theta: %s", grpnm);
    strcpy(sXlabel,"Time (ps)");
    strcpy(sYlabel,"\\f{Symbol}q\\f{} []"); // \f{Symbol} q \f{} :  = Symbol for theta
    fYmax=180.0;
    break;
  case ePlotPHI:
    sprintf(sTitle, "CP Phi: %s", grpnm);
    strcpy(sXlabel,"Time (ps)");
    strcpy(sYlabel,"\\f{Symbol}j\\f{} []"); // \f{Symbol} j \f{} :  = Symbol for phi
    fYmax=360.0;
    break;
  case ePlotQ:
    sprintf(sTitle, "CP puckering amplitude Q: %s", grpnm);
    strcpy(sXlabel,"Time (ps)");
    strcpy(sYlabel,"Q (nm)"); // \xC5 = Angstroem
    fYmax=0.1;
    break;
  default:
    gmx_fatal(FARGS,"Error: Type \"%s\" is not supported.\n",type);
    exit(1);
  } //end case


  out=xvgropen(fn, sTitle, sXlabel, sYlabel);
  if ( bPrintXvgrCodes() ) {
    fprintf(out,"@g0 on\n");
    fprintf(out,"@g0 type XY\n");
    fprintf(out,"@with g0\n");
    fprintf(out,"@    world 0, 0, %g, %g\n",fXmax, fYmax );
    fprintf(out,"@    xaxis  tick major 0.005\n");
    fprintf(out,"@    xaxis  tick minor ticks 1\n");
    fprintf(out,"@    yaxis  tick major 60\n");
    fprintf(out,"@    yaxis  tick minor ticks 1\n");
  }

  currentFrame=firstFrame;
  do {
    fprintf(out, "%10.5f", currentFrame->fTime);
    currentRing=currentFrame->rings;
    do {
      switch (type) {
        case ePlotTHETA:
          fprintf(out, "  %10.3f", currentRing->fTheta );
          break;
        case ePlotPHI:
          fprintf(out, "  %10.3f", currentRing->fPhi );
          break;
        case ePlotQ:
          fprintf(out, "  %10.8f", currentRing->fQ );
          break;
        default:
          break;
      }
      currentRing=currentRing->next;
    } while (currentRing);
    fprintf(out, "\n");
    currentFrame=currentFrame->next;
  } while (currentFrame);
  fclose(out);

}

void writePolarPlot(char *fn, char *grpnm, struct SFrame *firstFrame ) {
  /**
   * fn       filename
   * grpnm    Name of Indexgroup
   * firstFrame pointer to Linked List
   */
  char  sTitle[STRLEN];
  char  sXlabel[]="\\f{Symbol}j\\f{} []";// \f{Symbol} j \f{} :  = Symbol for phi
  char  sYlabel[]="\\f{Symbol}q\\f{} []";// \f{Symbol} q \f{} :  = Symbol for theta
  FILE  *out;
  struct SFrame *currentFrame;
  struct SRing  *currentRing;

  /* The Title */
  sprintf(sTitle, "CP Phi vs. Theta: %s", grpnm);
  out = xvgropen(fn, sTitle, sXlabel, sYlabel);
  fprintf(out,"@type xy\n");

  currentFrame=firstFrame;
  while (currentFrame) {
    currentRing=currentFrame->rings;
    while (currentRing) {
      fprintf(out, "%10.5f", currentRing->fPhi);
      fprintf(out, "%10.5f\n", currentRing->fTheta);
      currentRing=currentRing->next;
    }
    currentFrame=currentFrame->next;
  }
  fclose(out);

  /*** some old Version ***
   *
    sprintf(title, "CP Theta vs. Phi: %s", grpnm);
    out = xvgropen(opt2fn("-opol",NFILE,fnm), title,"\\f{Symbol}j\\f{} []", "\\f{Symbol}q\\f{} []");  // \f{Symbol} j \f{} :  = Symbol for phi
    xvgr_line_props(out,0,elNone,ecFrank);
    xvgr_view(out,0.2,0.2,0.8,0.8);
    xvgr_world(out,0,0,2*M_PI,M_PI);
    fprintf(out,"@    xaxis  tick on\n@    xaxis  tick major Pi/3\n@    xaxis  tick minor Pi/6\n");
    fprintf(out,"@    yaxis  tick on\n@    yaxis  tick major Pi/5\n@    yaxis  tick minor Pi/6\n");
    fprintf(out,"@ s0 symbol 2\n@ s0 symbol size 0.2\n@ s0 symbol fill 1\n");
    fprintf(out,"@TYPE XY\n");

    for (frame = 0; frame < nFrames; frame++) {
    //fprintf(out, "%10.3f", aTheta[frame][1]>180 ? aTheta[frame][1]-360 : aTheta[frame][1]  );
    //fprintf(out, "%10.3f\n", aPhi[frame][1]>180 ?   aPhi[frame][1]-360 :   aPhi[frame][1]   );
      fprintf(out, "%10.5f", aPhi[(frame*nRings)+1]/180*M_PI  );
      fprintf(out, "%10.5f\n", aTheta[(frame*nRings)+1]/180*M_PI  );
    //for (ring = 0; ring < nRings; ring++) {
    //  fprintf(out, "%10.3f", aTheta[frame][ring]);
    //  fprintf(out, "%10.3f\n", aPhi[frame][ring] );
    //}
   }
    fclose(out);
  *
  ***/
}

void writeCpDistributionPlot (char *fn, int type, char *grpnm, struct SFrame *firstFrame ) {
  /**
   * fn       filename
   * type     ???
   * grpnm    Name of Indexgroup
   * nFrames  Number of Frames
   * nRings   Number of Rings
   * aTheta   pointer to Array with Theta-Values
   * aPhi     pointer to Array with Phi-Values
   * aQ       pointer to Array with Q-Values
   * ...
   */
  /* Init */
  int   i, j=0;
  int   nValues=0;
  int   nMax=0;
  int   aDistFunc[360];
  float fXmax, fYmax=0;
  char  sTitle[STRLEN];
  char  sXlabel[STRLEN];
  char  sYlabel[STRLEN];
  FILE  *out;
  struct SFrame *currentFrame;
  struct SRing  *currentRing;
  strcpy(sYlabel,"relative probability [%]");

  switch (type) {
    case ePlotTHETA:
      fXmax=180.0;
      strcpy(sTitle, "Distribution of CP Angle Theta");
      strcpy(sXlabel,"\\f{Symbol}q\\f{} []"); // \f{Symbol} q \f{} :  = Symbol for theta
      break;
    case ePlotPHI:
      fXmax=360.0;
      strcpy(sTitle, "Distribution of CP Angle Phi");
      strcpy(sXlabel,"\\f{Symbol}j\\f{} []"); // \f{Symbol} j \f{} :  = Symbol for phi
      break;
    default:
      gmx_fatal(FARGS,"Error: Type is not supported.\n");
      exit(1);
      break;
  }


  for (i = 0; i < 360; ++i) {
    aDistFunc[i]=0;
  }

  /* Calculate Distribution Function :*/

  currentFrame=firstFrame;
  while (currentFrame) {
    currentRing=currentFrame->rings;
    while (currentRing) {
      nValues++;
      switch (type) {
        case ePlotTHETA:
          j=currentRing->fTheta+0.5;
          break;
        case ePlotPHI:
          j=currentRing->fPhi+0.5;
          break;
      }
      aDistFunc[j]++;
      if (aDistFunc[j]>nMax)
        nMax=aDistFunc[j];
      currentRing=currentRing->next;
    }//while currentRing
    currentFrame=currentFrame->next;
  }
  fYmax = nMax * 100.0 * 1.05;

  out=xvgropen(fn, sTitle, sXlabel, sYlabel);
  if ( bPrintXvgrCodes() ) {
    fprintf(out,"@g0 on\n");
    fprintf(out,"@g0 type XY\n");
    fprintf(out,"@with g0\n");
    fprintf(out,"@    world 0, 0, %g, %g\n",fXmax, fYmax );
    fprintf(out,"@    xaxis  tick major 60\n");
    fprintf(out,"@    xaxis  tick minor ticks 1\n");
    fprintf(out,"@    yaxis  tick major ticks 4\n");
    fprintf(out,"@    yaxis  tick minor ticks 1\n");
  }
  for (i = 0; i < 360; ++i) {
    if (aDistFunc[i]) {
      fprintf(out,"%4i %8.6f\n", i, ((float)aDistFunc[i]/nValues) * 100.0);
    }
  }

  fclose(out);
}

void selfTest() {
  float cptheta = 0.0; /* Just to make sure */
  float cpphi   = 0.0;
  float cpQ     = 0.0;
  int   ring, ringsize, atom;
  float coords_x[6], coords_y[6], coords_z[6];
  float testCoords[3][3][6] = {
    /* Sample Molecules from Cremer & Pople, JACS, 1975, 97(6), p. 1354*/
    { /* Furanoid Ring of Sucrose */
      {  0.0000,  1.1622,  0.7425, -0.7221, -1.1826, 0.0000 }, /* X coordinates in A */
      {  1.2111,  0.4349, -1.0012, -1.0309,  0.3861, 0.0000 }, /* Y coordinates in A */
      { -0.0189,  0.1461, -0.2174,  0.2057, -0.1154, 0.0000 }  /* Z coordinates in A */
      /* should give:
       * CREMER POPLE theta  =   n/a
       * CREMER POPLE phi2   =   265.1   []
       * CREMER POPLE Q      =     0.353 [A]
       */
    },
    { /* Pyranoid Ring of Sucrose */
      {  0.0000,  1.1997,  1.2356,  0.0110, -1.2300, -1.2164 }, /* X coordinates in A */
      {  1.3839,  0.7624, -0.7040, -1.4564, -0.7208,  0.7350 }, /* Y coordinates in A */
      {  0.1976, -0.2106,  0.2393, -0.2550,  0.2420, -0.2133 }  /* Z coordinates in A */
      /* should give:
       * CREMER POPLE theta  =     5.2   []
       * CREMER POPLE phi2   =   183.7   []
       * CREMER POPLE Q      =     0.556 [A]
       */
    },
    /* Sample Molecule from mdxview */
    { /* Pyranoid Ring of Clucose */
      { -1.323,  -0.509,   1.076,   1.301,   0.395,  -1.125  }, /* X coordinates in A */
      { -0.617,  -0.675,  -0.531,   0.688,   0.693,   0.521  }, /* Y coordinates in A */
      {  0.156,   1.337,   1.004,   0.186,  -1.070,  -0.701  }  /* Z coordinates in A */
      /* should give:
       * CREMER POPLE theta  =     4.48  []
       * CREMER POPLE phi2   =   167.38  []
       * CREMER POPLE Q      =     0.536 [A]
       */
    }
  };
  fprintf(stdout, "***************\n*  TEST DATA  *\n***************\n");
  for ( ring = 0; ring < asize(testCoords); ring++) {
    if (testCoords[ring][0][5]==0.0 && testCoords[ring][1][5]==0.0 && testCoords[ring][1][5]==0.0000)
      ringsize=5;
    else
      ringsize=6;

    for (atom = 0; atom < 6; atom++) {
      coords_x[atom] = testCoords[ring][0][atom];
      coords_y[atom] = testCoords[ring][1][atom];
      coords_z[atom] = testCoords[ring][2][atom];
      float dist=0.0;
      if (!(ringsize==5 && atom==5)) {
        dist = sqrt(
            ((coords_x[atom]-coords_x[(atom+1)%ringsize])*(coords_x[atom]-coords_x[(atom+1)%ringsize]))
          + ((coords_y[atom]-coords_y[(atom+1)%ringsize])*(coords_y[atom]-coords_y[(atom+1)%ringsize]))
          + ((coords_z[atom]-coords_z[(atom+1)%ringsize])*(coords_z[atom]-coords_z[(atom+1)%ringsize]))
            );
        fprintf(stdout, "Atom %i:\t%8.4f X, %8.4f Y, %8.4f Z, %8.4f\n", atom, coords_x[atom], coords_y[atom], coords_z[atom], dist);
      }
    }

    cremer_pople(coords_x, coords_y, coords_z, ringsize, &cptheta, &cpphi, &cpQ);
    fprintf(stdout, "CREMER POPLE theta  = %8.1f \n",cptheta);
    fprintf(stdout, "CREMER POPLE phi2   = %8.1f \n",cpphi);
    fprintf(stdout, "CREMER POPLE Q      = %8.5f \n",cpQ);
  }
}

void dumpFrames(struct SFrame *firstFrame) {
  struct SFrame *currentFrame;
  struct SRing  *currentRing;
  currentFrame=firstFrame;
  do  {
    printf("Frame No.: %5i                Time:%8.4f\n", currentFrame->nNumber, currentFrame->fTime);
    printf("=============================================\n");
    currentRing=currentFrame->rings;
    while (currentRing) {
      printf("Theta: %6.2f      Phi: %6.2f      Q: %6.4f\n", currentRing->fTheta, currentRing->fPhi, currentRing->fQ );
      currentRing=currentRing->next;
    }
    currentFrame=currentFrame->next;
  } while (currentFrame);
}

